@@ -26,7 +26,7 @@ JitterBuffer::JitterBuffer(MediaStreamItf *out, uint32_t step):bufferPool(JITTER
 		maxMinDelay=(uint32_t) ServerConfig::GetSharedInstance()->GetInt("jitter_max_delay_40", 15);
 		maxUsedSlots=(uint32_t) ServerConfig::GetSharedInstance()->GetInt("jitter_max_slots_40", 30);
 	}else{
-		minMinDelay=(uint32_t) ServerConfig::GetSharedInstance()->GetInt("jitter_min_delay_60", 1);
+		minMinDelay=(uint32_t) ServerConfig::GetSharedInstance()->GetInt("jitter_min_delay_60", 2);
 		maxMinDelay=(uint32_t) ServerConfig::GetSharedInstance()->GetInt("jitter_max_delay_60", 10);
 		maxUsedSlots=(uint32_t) ServerConfig::GetSharedInstance()->GetInt("jitter_max_slots_60", 20);
 	}
@@ -51,14 +51,14 @@ JitterBuffer::~JitterBuffer(){
 }
 
 void JitterBuffer::SetMinPacketCount(uint32_t count){
-	if(minDelay==count)
-		return;
+	LOGI("jitter: set min packet count %u", count);
 	minDelay=count;
+	minMinDelay=count;
 	//Reset();
 }
 
 int JitterBuffer::GetMinPacketCount(){
-	return minDelay;
+	return (int)minDelay;
 }
 
 size_t JitterBuffer::CallbackIn(unsigned char *data, size_t len, void *param){
@@ -110,6 +110,26 @@ size_t JitterBuffer::HandleOutput(unsigned char *buffer, size_t len, int offsetI
 	pkt.buffer=buffer;
 	pkt.size=len;
 	MutexGuard m(mutex);
+	if(first){
+		first=false;
+		unsigned int delay=GetCurrentDelay();
+		if(GetCurrentDelay()>5){
+			LOGW("jitter: delay too big upon start (%u), dropping packets", delay);
+			while(delay>GetMinPacketCount()){
+				for(int i=0;i<JITTER_SLOT_COUNT;i++){
+					if(slots[i].timestamp==nextTimestamp){
+						if(slots[i].buffer){
+    						bufferPool.Reuse(slots[i].buffer);
+    						slots[i].buffer=NULL;
+						}
+						break;
+					}
+				}
+				Advance();
+				delay--;
+			}
+		}
+	}
 	int result=GetInternal(&pkt, offsetInSteps, advance);
 	if(outstandingDelayChange!=0){
 		if(outstandingDelayChange<0){
@@ -173,7 +193,7 @@ int JitterBuffer::GetInternal(jitter_packet_t* pkt, int offset, bool advance){
 		return JR_OK;
 	}
 
-	//LOGV("jitter: found no packet for timestamp %lld (last put = %d, lost = %d)", (long long int)timestampToGet, lastPutTimestamp, lostCount);
+	LOGV("jitter: found no packet for timestamp %lld (last put = %d, lost = %d)", (long long int)timestampToGet, lastPutTimestamp, lostCount);
 
 	if(advance)
 		Advance();
@@ -190,7 +210,7 @@ int JitterBuffer::GetInternal(jitter_packet_t* pkt, int offset, bool advance){
 			dontIncMinDelay=16;
 			dontDecMinDelay+=128;
 			if(GetCurrentDelay()<minDelay)
-				nextTimestamp-=(minDelay-GetCurrentDelay());
+				nextTimestamp-=(int64_t)(minDelay-GetCurrentDelay());
 			lostCount=0;
 			Reset();
 		}
@@ -222,7 +242,8 @@ void JitterBuffer::PutInternal(jitter_packet_t* pkt, bool overwriteExisting){
 	if(wasReset){
 		wasReset=false;
 		outstandingDelayChange=0;
-		nextTimestamp=((int64_t)pkt->timestamp)-step*minDelay;
+		nextTimestamp=(int64_t)(((int64_t)pkt->timestamp)-step*minDelay);
+		first=true;
 		LOGI("jitter: resyncing, next timestamp = %lld (step=%d, minDelay=%f)", (long long int)nextTimestamp, step, minDelay);
 	}
 	
@@ -351,7 +372,7 @@ void JitterBuffer::Tick(){
 	if(stddevDelay>maxMinDelay)
 		stddevDelay=maxMinDelay;
 	if(stddevDelay!=minDelay){
-		int32_t diff=stddevDelay-minDelay;
+		int32_t diff=(int32_t)(stddevDelay-minDelay);
 		if(diff>0){
 			dontDecMinDelay=100;
 		}
