@@ -6,6 +6,7 @@
 
 #include "OpusEncoder.h"
 #include <assert.h>
+#include <algorithm>
 #include "logging.h"
 #include "VoIPServerConfig.h"
 #ifdef HAVE_CONFIG_H
@@ -23,32 +24,29 @@ tgvoip::OpusEncoder::OpusEncoder(MediaStreamItf *source, bool needSecondary):que
 	opus_encoder_ctl(enc, OPUS_SET_INBAND_FEC(1));
 	opus_encoder_ctl(enc, OPUS_SET_SIGNAL(OPUS_SIGNAL_VOICE));
 	opus_encoder_ctl(enc, OPUS_SET_BANDWIDTH(OPUS_BANDWIDTH_FULLBAND));
-	requestedBitrate=32000;
+	requestedBitrate=20000;
 	currentBitrate=0;
 	running=false;
 	echoCanceller=NULL;
 	complexity=10;
 	frameDuration=20;
 	levelMeter=NULL;
-	mediumCorrectionBitrate=ServerConfig::GetSharedInstance()->GetInt("audio_medium_fec_bitrate", 10000);
-	strongCorrectionBitrate=ServerConfig::GetSharedInstance()->GetInt("audio_strong_fec_bitrate", 8000);
-	mediumCorrectionMultiplier=ServerConfig::GetSharedInstance()->GetDouble("audio_medium_fec_multiplier", 1.5);
-	strongCorrectionMultiplier=ServerConfig::GetSharedInstance()->GetDouble("audio_strong_fec_multiplier", 2.0);
+	mediumCorrectionBitrate=static_cast<uint32_t>(ServerConfig::GetSharedInstance()->GetInt("audio_medium_fec_bitrate", 10000));
+	strongCorrectionBitrate=static_cast<uint32_t>(ServerConfig::GetSharedInstance()->GetInt("audio_strong_fec_bitrate", 8000));
+	mediumCorrectionMultiplier=ServerConfig::GetSharedInstance()->GetDouble("audio_medium_fec_multiplier", 0.8);
+	strongCorrectionMultiplier=ServerConfig::GetSharedInstance()->GetDouble("audio_strong_fec_multiplier", 0.5);
+	vadNoVoiceBitrate=static_cast<uint32_t>(ServerConfig::GetSharedInstance()->GetInt("audio_vad_no_voice_bitrate", 6000));
 	secondaryEncoderEnabled=false;
 
 	if(needSecondary){
 		secondaryEncoder=opus_encoder_create(48000, 1, OPUS_APPLICATION_VOIP, NULL);
 		opus_encoder_ctl(secondaryEncoder, OPUS_SET_COMPLEXITY(10));
 		opus_encoder_ctl(secondaryEncoder, OPUS_SET_SIGNAL(OPUS_SIGNAL_VOICE));
-		opus_encoder_ctl(secondaryEncoder, OPUS_SET_VBR(0));
-		opus_encoder_ctl(secondaryEncoder, OPUS_SET_BANDWIDTH(OPUS_BANDWIDTH_FULLBAND));
+		//opus_encoder_ctl(secondaryEncoder, OPUS_SET_VBR(0));
 		opus_encoder_ctl(secondaryEncoder, OPUS_SET_BITRATE(8000));
 		opus_encoder_ctl(secondaryEncoder, OPUS_SET_INBAND_FEC(1));
 		opus_encoder_ctl(secondaryEncoder, OPUS_SET_PACKET_LOSS_PERC(15));
-
-		opus_int32 delay, ecDelay;
-		opus_encoder_ctl(secondaryEncoder, OPUS_GET_LOOKAHEAD(&ecDelay));
-		opus_encoder_ctl(enc, OPUS_GET_LOOKAHEAD(&delay));
+		opus_encoder_ctl(secondaryEncoder, OPUS_SET_BANDWIDTH(OPUS_BANDWIDTH_SUPERWIDEBAND));
 	}else{
 		secondaryEncoder=NULL;
 	}
@@ -64,7 +62,7 @@ void tgvoip::OpusEncoder::Start(){
 	if(running)
 		return;
 	running=true;
-	thread=new Thread(new MethodPointer<tgvoip::OpusEncoder>(&tgvoip::OpusEncoder::RunThread, this), NULL);
+	thread=new Thread(std::bind(&tgvoip::OpusEncoder::RunThread, this));
 	thread->SetName("OpusEncoder");
 	thread->Start();
 	thread->SetMaxPriority();
@@ -84,15 +82,15 @@ void tgvoip::OpusEncoder::SetBitrate(uint32_t bitrate){
 	requestedBitrate=bitrate;
 }
 
-void tgvoip::OpusEncoder::Encode(unsigned char *data, size_t len){
+void tgvoip::OpusEncoder::Encode(int16_t* data, size_t len){
 	if(requestedBitrate!=currentBitrate){
 		opus_encoder_ctl(enc, OPUS_SET_BITRATE(requestedBitrate));
 		currentBitrate=requestedBitrate;
 		LOGV("opus_encoder: setting bitrate to %u", currentBitrate);
 	}
 	if(levelMeter)
-		levelMeter->Update(reinterpret_cast<int16_t *>(data), len/2);
-	int32_t r=opus_encode(enc, (int16_t*)data, len/2, buffer, 4096);
+		levelMeter->Update(data, len);
+	int32_t r=opus_encode(enc, data, static_cast<int>(len), buffer, 4096);
 	if(r<=0){
 		LOGE("Error encoding: %d", r);
 	}else if(r==1){
@@ -102,7 +100,7 @@ void tgvoip::OpusEncoder::Encode(unsigned char *data, size_t len){
 		int32_t secondaryLen=0;
 		unsigned char secondaryBuffer[128];
 		if(secondaryEncoderEnabled && secondaryEncoder){
-			secondaryLen=opus_encode(secondaryEncoder, (int16_t*)data, len/2, secondaryBuffer, sizeof(secondaryBuffer));
+			secondaryLen=opus_encode(secondaryEncoder, data, static_cast<int>(len), secondaryBuffer, sizeof(secondaryBuffer));
 			//LOGV("secondaryLen %d", secondaryLen);
 		}
 		InvokeCallback(buffer, (size_t)r, secondaryBuffer, (size_t)secondaryLen);
@@ -135,34 +133,67 @@ void tgvoip::OpusEncoder::SetEchoCanceller(EchoCanceller* aec){
 	echoCanceller=aec;
 }
 
-void tgvoip::OpusEncoder::RunThread(void* arg){
-	unsigned char buf[960*2];
+void tgvoip::OpusEncoder::RunThread(){
 	uint32_t bufferedCount=0;
 	uint32_t packetsPerFrame=frameDuration/20;
 	LOGV("starting encoder, packets per frame=%d", packetsPerFrame);
-	unsigned char* frame;
+	int16_t* frame;
 	if(packetsPerFrame>1)
-		frame=(unsigned char *) malloc(960*2*packetsPerFrame);
+		frame=(int16_t*) malloc(960*2*packetsPerFrame);
 	else
 		frame=NULL;
+	bool frameHasVoice=false;
+	bool wasVadMode=false;
 	while(running){
-		unsigned char* packet=(unsigned char*)queue.GetBlocking();
+		int16_t* packet=(int16_t*)queue.GetBlocking();
 		if(packet){
+			bool hasVoice=true;
 			if(echoCanceller)
-				echoCanceller->ProcessInput(packet, buf, 960*2);
-			else
-				memcpy(buf, packet, 960*2);
-			if(packetsPerFrame==1){
-				Encode(buf, 960*2);
-			}else{
-				memcpy(frame+(960*2*bufferedCount), buf, 960*2);
-				bufferedCount++;
-				if(bufferedCount==packetsPerFrame){
-					Encode(frame, 960*2*packetsPerFrame);
-					bufferedCount=0;
+				echoCanceller->ProcessInput(packet, 960, hasVoice);
+			if(!postProcEffects.empty()){
+				for(effects::AudioEffect* effect:postProcEffects){
+					effect->Process(packet, 960);
 				}
 			}
-			bufferPool.Reuse(packet);
+			if(packetsPerFrame==1){
+				Encode(packet, 960);
+			}else{
+				memcpy(frame+(960*bufferedCount), packet, 960*2);
+    			frameHasVoice=frameHasVoice || hasVoice;
+				bufferedCount++;
+				if(bufferedCount==packetsPerFrame){
+					if(vadMode){
+						if(frameHasVoice){
+							opus_encoder_ctl(enc, OPUS_SET_BITRATE(currentBitrate));
+							opus_encoder_ctl(enc, OPUS_SET_BANDWIDTH(OPUS_BANDWIDTH_SUPERWIDEBAND));
+							if(secondaryEncoder){
+								opus_encoder_ctl(secondaryEncoder, OPUS_SET_BITRATE(currentBitrate));
+								opus_encoder_ctl(secondaryEncoder, OPUS_SET_BANDWIDTH(OPUS_BANDWIDTH_SUPERWIDEBAND));
+							}
+						}else{
+							opus_encoder_ctl(enc, OPUS_SET_BITRATE(vadNoVoiceBitrate));
+							opus_encoder_ctl(enc, OPUS_SET_BANDWIDTH(OPUS_BANDWIDTH_NARROWBAND));
+							if(secondaryEncoder){
+								opus_encoder_ctl(secondaryEncoder, OPUS_SET_BITRATE(vadNoVoiceBitrate));
+								opus_encoder_ctl(secondaryEncoder, OPUS_SET_BANDWIDTH(OPUS_BANDWIDTH_NARROWBAND));
+							}
+						}
+						wasVadMode=true;
+					}else if(wasVadMode){
+						wasVadMode=false;
+						opus_encoder_ctl(enc, OPUS_SET_BITRATE(currentBitrate));
+						opus_encoder_ctl(enc, OPUS_SET_BANDWIDTH(OPUS_BANDWIDTH_FULLBAND));
+						if(secondaryEncoder){
+							opus_encoder_ctl(secondaryEncoder, OPUS_SET_BITRATE(currentBitrate));
+							opus_encoder_ctl(secondaryEncoder, OPUS_SET_BANDWIDTH(OPUS_BANDWIDTH_SUPERWIDEBAND));
+						}
+					}
+					Encode(frame, 960*packetsPerFrame);
+					bufferedCount=0;
+					frameHasVoice=false;
+				}
+			}
+			bufferPool.Reuse(reinterpret_cast<unsigned char *>(packet));
 		}
 	}
 	if(frame)
@@ -176,14 +207,13 @@ void tgvoip::OpusEncoder::SetOutputFrameDuration(uint32_t duration){
 
 
 void tgvoip::OpusEncoder::SetPacketLoss(int percent){
-	packetLossPercent=percent;
+	packetLossPercent=std::min(20, percent);
 	double multiplier=1;
 	if(currentBitrate<=strongCorrectionBitrate)
 		multiplier=strongCorrectionMultiplier;
 	else if(currentBitrate<=mediumCorrectionBitrate)
 		multiplier=mediumCorrectionMultiplier;
 	opus_encoder_ctl(enc, OPUS_SET_PACKET_LOSS_PERC((int)(percent*multiplier)));
-	opus_encoder_ctl(enc, OPUS_SET_BANDWIDTH(percent>17 ? OPUS_AUTO : OPUS_BANDWIDTH_FULLBAND));
 }
 
 int tgvoip::OpusEncoder::GetPacketLoss(){
@@ -210,3 +240,16 @@ void tgvoip::OpusEncoder::InvokeCallback(unsigned char *data, size_t length, uns
 void tgvoip::OpusEncoder::SetSecondaryEncoderEnabled(bool enabled){
 	secondaryEncoderEnabled=enabled;
 }
+
+void tgvoip::OpusEncoder::SetVadMode(bool vad){
+	vadMode=vad;
+}
+void tgvoip::OpusEncoder::AddAudioEffect(effects::AudioEffect *effect){
+	postProcEffects.push_back(effect);
+}
+
+void tgvoip::OpusEncoder::RemoveAudioEffect(effects::AudioEffect *effect){
+	std::vector<effects::AudioEffect*>::iterator i=std::find(postProcEffects.begin(), postProcEffects.end(), effect);
+	if(i!=postProcEffects.end())
+		postProcEffects.erase(i);
+}
