@@ -13,6 +13,7 @@
 #include "BlockingQueue.h"
 #include "Buffers.h"
 #include "EchoCanceller.h"
+#include "utils.h"
 
 #include <stdint.h>
 
@@ -21,6 +22,7 @@ struct OpusEncoder;
 namespace tgvoip{
 class OpusEncoder{
 public:
+	TGVOIP_DISALLOW_COPY_AND_ASSIGN(OpusEncoder);
 	OpusEncoder(MediaStreamItf* source, bool needSecondary);
 	virtual ~OpusEncoder();
 	virtual void Start();
@@ -35,11 +37,14 @@ public:
 	void SetLevelMeter(AudioLevelMeter* levelMeter);
 	void SetCallback(void (*f)(unsigned char*, size_t, unsigned char*, size_t, void*), void* param);
 	void SetSecondaryEncoderEnabled(bool enabled);
+	void SetVadMode(bool vad);
+	void AddAudioEffect(effects::AudioEffect* effect);
+	void RemoveAudioEffect(effects::AudioEffect* effect);
 
 private:
 	static size_t Callback(unsigned char* data, size_t len, void* param);
-	void RunThread(void* arg);
-	void Encode(unsigned char* data, size_t len);
+	void RunThread();
+	void Encode(int16_t* data, size_t len);
 	void InvokeCallback(unsigned char* data, size_t length, unsigned char* secondaryData, size_t secondaryLength);
 	MediaStreamItf* source;
 	::OpusEncoder* enc;
@@ -61,6 +66,9 @@ private:
 	double strongCorrectionMultiplier;
 	AudioLevelMeter* levelMeter;
 	bool secondaryEncoderEnabled;
+	bool vadMode=false;
+	uint32_t vadNoVoiceBitrate;
+	std::vector<effects::AudioEffect*> postProcEffects;
 
 	void (*callback)(unsigned char*, size_t, unsigned char*, size_t, void*);
 	void* callbackParam;
